//D3D window class implementation

#include "d3dwindow.h"

//Set new instances of the d3d app and device contexts to null
D3DWindow* gd3dWin = NULL;
IDirect3DDevice9* gd3dDevice = NULL;

/////////////////////////////////////////////
//D3DWindow class function implementations
/////////////////////////////////////////////

//Make sure application is created before processing messeges
LRESULT CALLBACK MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	if(gd3dWin != 0)
	{
		return gd3dWin->WndProc(msg, wParam, lParam);
	}
	else
	{
		return DefWindowProc(hwnd, msg, wParam, lParam);
	}
}

//D3DWindow constructor implementation
D3DWindow::D3DWindow(HINSTANCE hInstance, std::string winCaption, D3DDEVTYPE devType, DWORD requestedVP)
{
	mMainWinName = winCaption;
	mDevType = devType;
	mRequestedVP = requestedVP;
	hmAppInst = hInstance;
	hmMainWin = NULL;
	md3dObj = NULL;
	mPaused = false;
	ZeroMemory(&md3dPP, sizeof(md3dPP));

	initWindow();
	initD3D();
}

//D3DWindow destructor implementation
D3DWindow::~D3DWindow()
{
	Release(md3dObj);
	Release(gd3dDevice);
}

//Get app instance implementation
HINSTANCE D3DWindow::getAppInst()
{
	return hmAppInst;
}

//Get window handle implementation
HWND D3DWindow::getWinHwnd()
{
	return hmMainWin;
}

//Window initialization implementation
void D3DWindow::initWindow()
{
	WNDCLASSEX  wcex;									

    wcex.cbSize        = sizeof (wcex);				
    wcex.style         = CS_HREDRAW | CS_VREDRAW;		
    wcex.lpfnWndProc   = MainWndProc;						
    wcex.cbClsExtra    = 0;								
    wcex.cbWndExtra    = 0;								
    wcex.hInstance     = hmAppInst;						
    wcex.hIcon         = 0; 
    wcex.hCursor       = LoadCursor (NULL, IDC_ARROW);	
															
    wcex.hbrBackground = (HBRUSH) (COLOR_WINDOW+1);
	wcex.lpszMenuName  = NULL; 					
    wcex.lpszClassName = "Framework Test";				
    wcex.hIconSm       = 0; 

	RegisterClassEx (&wcex);	

	RECT R = {0, 0, 800, 600};
	AdjustWindowRect(&R, WS_OVERLAPPEDWINDOW, false);

	hmMainWin = CreateWindow("Framework Test", mMainWinName.c_str(), 
							WS_OVERLAPPEDWINDOW, 100, 100, R.right, R.bottom, 
							0, 0, hmAppInst, 0); 

	ShowWindow(hmMainWin, SW_SHOW);
	UpdateWindow(hmMainWin);
}

//D3D initialization implementation
void D3DWindow::initD3D()
{
	// Step 1: Create the IDirect3D9 object.

    md3dObj = Direct3DCreate9(D3D_SDK_VERSION);
	if(!md3dObj)
	{
		MessageBox(0, "Direct3DCreate9 FAILED", 0, 0);
		PostQuitMessage(0);
	}


	// Step 2: Verify hardware support for specified formats in windowed and full screen modes.
	
	D3DDISPLAYMODE mode;
	md3dObj->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &mode);
	HR(md3dObj->CheckDeviceType(D3DADAPTER_DEFAULT, mDevType, mode.Format, mode.Format, true));
	HR(md3dObj->CheckDeviceType(D3DADAPTER_DEFAULT, mDevType, D3DFMT_X8R8G8B8, D3DFMT_X8R8G8B8, false));

	// Step 3: Check for requested vertex processing and pure device.

	D3DCAPS9 caps;
	HR(md3dObj->GetDeviceCaps(D3DADAPTER_DEFAULT, mDevType, &caps));

	DWORD devBehaviorFlags = 0;
	if( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT )
		devBehaviorFlags |= mRequestedVP;
	else
		devBehaviorFlags |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;

	// If pure device and HW T&L supported
	if( caps.DevCaps & D3DDEVCAPS_PUREDEVICE &&
		devBehaviorFlags & D3DCREATE_HARDWARE_VERTEXPROCESSING)
			devBehaviorFlags |= D3DCREATE_PUREDEVICE;

	// Step 4: Fill out the D3DPRESENT_PARAMETERS structure.

	md3dPP.BackBufferWidth            = 0; 
	md3dPP.BackBufferHeight           = 0;
	md3dPP.BackBufferFormat           = D3DFMT_UNKNOWN;
	md3dPP.BackBufferCount            = 1;
	md3dPP.MultiSampleType            = D3DMULTISAMPLE_NONE;
	md3dPP.MultiSampleQuality         = 0;
	md3dPP.SwapEffect                 = D3DSWAPEFFECT_DISCARD; 
	md3dPP.hDeviceWindow              = hmMainWin;
	md3dPP.Windowed                   = true;
	md3dPP.EnableAutoDepthStencil     = true; 
	md3dPP.AutoDepthStencilFormat     = D3DFMT_D24S8;
	md3dPP.Flags                      = 0;
	md3dPP.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	md3dPP.PresentationInterval       = D3DPRESENT_INTERVAL_IMMEDIATE;


	// Step 5: Create the device.

	HR(md3dObj->CreateDevice(
		D3DADAPTER_DEFAULT, // primary adapter
		mDevType,           // device type
		hmMainWin,          // window associated with device
		devBehaviorFlags,   // vertex processing
	    &md3dPP,            // present parameters
	    &gd3dDevice));      // return created device
}

//Run app implementation
int D3DWindow::Run()
{
	//Window message pump goes here
	MSG  msg;
	msg.message = WM_NULL;

	//Application timer implementation
		__int64 cntsPerSec = 0;
	QueryPerformanceFrequency((LARGE_INTEGER*)&cntsPerSec);
	float secsPerCnt = 1.0f / (float)cntsPerSec;

	__int64 prevTimeStamp = 0;
	QueryPerformanceCounter((LARGE_INTEGER*)&prevTimeStamp);

	while(msg.message != WM_QUIT)
    {
		// If there are Window messages then process them.
		if(PeekMessage( &msg, 0, 0, 0, PM_REMOVE ))
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		//Run game
		else
		{	
			if(mPaused)
			{
				Sleep(20);
				continue;
			}
			if(!isDeviceLost())
			{
				//calculate current time stap
				__int64 currTimeStamp = 0;
				QueryPerformanceCounter((LARGE_INTEGER*)&currTimeStamp);
				float dt = (currTimeStamp - prevTimeStamp)*secsPerCnt;

				Update(dt);
				Draw();

				//calculate next iteration
				prevTimeStamp = currTimeStamp;
			}
		}
    }
	return (int)msg.wParam;
}

//App message proc implementation
LRESULT D3DWindow::WndProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	// Is the application in a minimized or maximized state?
	static bool minOrMaxed = false;

	RECT clientRect = {0, 0, 0, 0};
	switch(msg)
	{
	case WM_ACTIVATE:
		if(LOWORD(wParam) == WA_INACTIVE)
		{
			mPaused = true;
		}
		else
		{
			mPaused = false;
		}
		return 0;


	// WM_SIZE is sent when the user resizes the window.  
	case WM_SIZE:
		if(gd3dDevice)
		{
			md3dPP.BackBufferWidth  = LOWORD(lParam);
			md3dPP.BackBufferHeight = HIWORD(lParam);

			if(wParam == SIZE_MINIMIZED)
			{
				mPaused = true;
				minOrMaxed = true;
			}
			else if(wParam == SIZE_MAXIMIZED)
			{
				mPaused = false;
				minOrMaxed = true;
				onLostDevice();
				HR(gd3dDevice->Reset(&md3dPP));
				onResetDevice();
			}
			else if(wParam == SIZE_RESTORED)
			{
				mPaused = false;

				if(minOrMaxed && md3dPP.Windowed)
				{
					onLostDevice();
					HR(gd3dDevice->Reset(&md3dPP));
					onResetDevice();
				}
				else
				{
					//nothing
				}
				minOrMaxed = false;
			}
		}
		return 0;


	// WM_EXITSIZEMOVE is sent when the user releases the resize bars.
	// Here we reset everything based on the new window dimensions.
	case WM_EXITSIZEMOVE:
		GetClientRect(hmMainWin, &clientRect);
		md3dPP.BackBufferWidth  = clientRect.right;
		md3dPP.BackBufferHeight = clientRect.bottom;
		onLostDevice();
		HR(gd3dDevice->Reset(&md3dPP));
		onResetDevice();

		return 0;

	// WM_CLOSE is sent when the user presses the 'X' button in the
	// caption bar menu.
	case WM_CLOSE:
		DestroyWindow(hmMainWin);
		return 0;

	// WM_DESTROY is sent when the window is being destroyed.
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;

		//Note the lack of DirectInput here. This was just hacked in really quickly. 
	case WM_KEYDOWN:
		if(wParam == VK_ESCAPE)
		{
			goFullscreen(false);
		}
		else if(wParam == 'F')
		{
			goFullscreen(true);
		}
		return 0;
	}
	return DefWindowProc(hmMainWin, msg, wParam, lParam);
}

//Window fullscreen implementation

//Todo: Change the DirectInput Cooperative level from DISCL_FOREGROUND to DISCL_EXCLUSIVE if application is running fullscreen.
void D3DWindow::goFullscreen(bool enable)
{
	// Switch to fullscreen mode.
	if(enable)
	{
		if(!md3dPP.Windowed) 
			return;

		int width  = GetSystemMetrics(SM_CXSCREEN);
		int height = GetSystemMetrics(SM_CYSCREEN);

		md3dPP.BackBufferFormat = D3DFMT_X8R8G8B8;
		md3dPP.BackBufferWidth  = width;
		md3dPP.BackBufferHeight = height;
		md3dPP.Windowed         = false;

		// Change the window style to a more fullscreen friendly style.
		SetWindowLongPtr(hmMainWin, GWL_STYLE, WS_POPUP);

		SetWindowPos(hmMainWin, HWND_TOP, 0, 0, width, height, SWP_NOZORDER | SWP_SHOWWINDOW);	
	}
	// Switch to windowed mode.
	else
	{
		// Are we already in windowed mode?
		if( md3dPP.Windowed ) 
			return;

		RECT R = {0, 0, 800, 600};
		AdjustWindowRect(&R, WS_OVERLAPPEDWINDOW, false);
		md3dPP.BackBufferFormat = D3DFMT_UNKNOWN;
		md3dPP.BackBufferWidth  = 800;
		md3dPP.BackBufferHeight = 600;
		md3dPP.Windowed         = true;
	
		// Change the window style to a more windowed friendly style.
		SetWindowLongPtr(hmMainWin, GWL_STYLE, WS_OVERLAPPEDWINDOW);


		SetWindowPos(hmMainWin, HWND_TOP, 100, 100, R.right, R.bottom, SWP_NOZORDER | SWP_SHOWWINDOW);
	}

	// Reset the device with the changes.
	onLostDevice();
	HR(gd3dDevice->Reset(&md3dPP));
	onResetDevice();	
}

//Incase device is lost implementation
bool D3DWindow::isDeviceLost()
{
	// Get the state of the graphics device.
	HRESULT hr = gd3dDevice->TestCooperativeLevel();

	if(hr == D3DERR_DEVICELOST)
	{
		Sleep(20);
		return true;
	}
	// Driver error, exit.
	else if(hr == D3DERR_DRIVERINTERNALERROR)
	{
		MessageBox(0, "Internal Driver Error...Exiting", 0, 0);
		PostQuitMessage(0);
		return true;
	}
	// The device is lost but we can reset and restore it.
	else if(hr == D3DERR_DEVICENOTRESET)
	{
		onLostDevice();
		HR(gd3dDevice->Reset(&md3dPP));
		onResetDevice();
		return false;
	}
	else
	{
		return false;
	}
}

