//D3D specific windowing class interface
//By Vladeta Stojanovic (0602920@live.abertay.ac.uk)

#ifndef _D3DWINDOW_H_
#define _D3DWINDOW_H_

//system includes
#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h> 
//#include <dxerr9.h> 
#include <DxErr.h>
#include <tchar.h>
#include <crtdbg.h>
//#include <string>
#include <sstream>
#include <vector>
#include <list>


/////////////////////////////
//Memory managment macros
////////////////////////////
#define Delete(pointer)	{if(pointer){delete(pointer); pointer = NULL;}}
#define DeleteArray(pointer)	{if(pointer){delete[](pointer); pointer = NULL;}}
#define Release(pointer)	{if(pointer){pointer->Release(); pointer = NULL;}}

/////////
//Debug
////////
#if defined(DEBUG) | defined(_DEBUG)
	#ifndef HR
	#define HR(x)                                      \
	{                                                  \
		HRESULT hr = x;                                \
		if(FAILED(hr))                                 \
		{                                              \
			DXTrace(__FILE__, __LINE__, hr, #x, TRUE); \
		}                                              \
	}
	#endif

#else
	#ifndef HR
	#define HR(x) x;
	#endif
#endif 

//The D3DWinodw class 
class D3DWindow
{
public:
	//d3d window class constructor
	D3DWindow(HINSTANCE hInstance, std::string winCaption, D3DDEVTYPE devType, DWORD requestedVP);
	
	//destructor
	virtual ~D3DWindow();

	//return app istance
	HINSTANCE getAppInst();

	//get window handle
	HWND getWinHwnd();

	///////////////////////////////////////
	//Window control function prototypes
	///////////////////////////////////////

	//Can overide
	virtual bool checkDeviceCaps()
	{
		return true;
	}

	virtual void onLostDevice()	{}
	virtual void onResetDevice() {}
	virtual void Update(float fTime) {}
	virtual void Draw() {}

	//Probably wont need to overide
	void initWindow();
	void initD3D();
	int Run();
	LRESULT CALLBACK WndProc(UINT msg, WPARAM wParam, LPARAM lParam);
	void goFullscreen(bool enable);
	bool isDeviceLost();
	
protected:
	//derived class can inherit and modify window behaviour attributes
	std::string mMainWinName;
	D3DDEVTYPE mDevType;
	DWORD mRequestedVP;
	
	//D3D and Windows specific data members
	HINSTANCE hmAppInst;
	HWND hmMainWin;
	IDirect3D9* md3dObj;
	bool mPaused;
	D3DPRESENT_PARAMETERS md3dPP;
};

//externed variables 
extern D3DWindow* gd3dWin;
extern IDirect3DDevice9* gd3dDevice;

#endif _WINDOW_H_
