//Game input class implementation

#include "input.h"

//Todo: Enable debug functionality by casting all the initialization functions with HR()

Input* gGameInput = NULL;

//Input class constructor implementation
Input::Input(HWND gameWindow)
{
	//set the handle to current game window 
	m_inputWindow = gameWindow;

	//Create DirectInput interface
	DirectInput8Create(GetModuleHandle( NULL ), DIRECTINPUT_VERSION, IID_IDirectInput8, (void**)&m_diObj, NULL);

	//Create and acquire keyboard device
	m_diObj->CreateDevice(GUID_SysKeyboard, &m_keyboard, NULL);
	m_keyboard->SetDataFormat(&c_dfDIKeyboard);
	m_keyboard->SetCooperativeLevel(m_inputWindow, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
	m_keyboard->Acquire();

	//Create and acquire mouse device
	m_diObj->CreateDevice(GUID_SysMouse, &m_mouse, NULL);
	m_mouse->SetDataFormat(&c_dfDIMouse);
	m_mouse->SetCooperativeLevel(m_inputWindow, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE);
	m_mouse->Acquire();

	//Init the press stamp (used to keep track of when each key/button was last pressed)
	m_pressStamp = 0;
}

//Input class destructor implementation
Input::~Input()
{
	Release(m_diObj);
	Release(m_keyboard);
	Release(m_mouse);
}

//Update input function implementation
void Input::UpdateInput()
{
	//Input device poll result 
	static HRESULT inputResult;

	//Poll keyboard (abort if unkown error occurs)
	while(true)
	{
		m_keyboard->Poll();

		if(SUCCEEDED(inputResult = m_keyboard->GetDeviceState(256, (LPVOID)&m_keyState)))
		{
			break;
		}
		if(inputResult != DIERR_INPUTLOST && inputResult != DIERR_NOTACQUIRED)
		{
			return;
		}

		// Reacquire the device if the focus was lost.
		if(FAILED( m_keyboard->Acquire()))
		{
			return;
		}
	}

	//Poll mouse (abort if unkown error occurs)
	while(true)
	{
		m_mouse->Poll();

		if(SUCCEEDED(inputResult = m_mouse->GetDeviceState(sizeof(DIMOUSESTATE), &m_mouseState)))
		{
			break;
		}
		if(inputResult != DIERR_INPUTLOST && inputResult != DIERR_NOTACQUIRED)
		{
			return;
		}

		// Reacquire the device if the focus was lost.
		if(FAILED(m_mouse->Acquire()))
		{
			return;
		}
	}

	//Get position of mouse
	GetCursorPos(&m_position);
	ScreenToClient(m_inputWindow, &m_position);

	//Increment press stamp
	m_pressStamp++;
}

//Get key press function implementation
bool Input::GetKeyPress(char key, bool ignorePressStamp)
{
	bool pressed = true;

	if((m_keyState[key] & 0x80) == false)
	{
		return false;
	}

	if(ignorePressStamp == false)
	{
		if(m_keyPressStamp[key] == m_pressStamp - 1 || m_keyPressStamp[key] == m_pressStamp)
		{
			pressed = false;
		}
	}

	m_keyPressStamp[key] = m_pressStamp;

	return pressed;
}

//Mouse get button press function implementation
bool Input::GetButtonPress(char button, bool ignorePressStamp)
{
	bool pressed = true;

	if((m_mouseState.rgbButtons[button] & 0x80) == false)
	{
		return false;
	}

	if(ignorePressStamp == false)
	{
		if(m_buttonPressStamp[button] == m_pressStamp - 1 || m_buttonPressStamp[button] == m_pressStamp)
		{
			pressed = false;
		}
	}

	m_buttonPressStamp[button] = m_pressStamp;

	return pressed;
}

//Get mouse pos x
long Input::GetMousePosX()
{
	return m_position.x;
}

//Get mouse pos y
long Input::GetMousePosY()
{
	return m_position.y;
}

//Get mouse delta x
long Input::GetMouseDeltaX()
{
	return m_mouseState.lX;
}

//Get mouse delta y
long Input::GetMouseDeltaY()
{
	return m_mouseState.lY;
}

//Get mouse wheel delta
long Input::GetMouseWheel()
{
	return m_mouseState.lZ;
}

