////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Basic .x file mesh handling class.
// Static meshes only for the moment.
// Written by Vladeta Stojanovic (0602920@live.abertay.ac.uk)
// Based on code examples from Introduction to 3D Game Programming with DirectX 9.0c, by Frank D. Luna
///////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef _MESH_H_
#define _MESH_H_

#include "scene.h"

class XMesh
{
public:

	//Class constructor
	XMesh(void);

	//Class destructor
	~XMesh();

	//.x file loading routine
	void LoadXFile(const std::string& filename, 
					ID3DXMesh** meshOut,
					std::vector<Mtrl>& mtrls, 
					std::vector<IDirect3DTexture9*>& texs);

	void GenTexCoords(float texScale);

	//Load and initialize mesh
	void LoadandInitDiffMap(char * meshFile, char * meshTex,float texScale_);
	void LoadandInitEnvMap(char * meshFile, char * meshTex, char * envTex, float texScale_);
	void LoadandInitNMMap(char * meshFile, char * meshTex, char * nmTex);

	//Draw mesh with diffuse tex only
	void DrawDiffuse(D3DXMATRIX* pMatrix, ID3DXEffect* pFX, D3DXHANDLE phMtrl, D3DXHANDLE phTex);

	//Draw mesh with diffuse + normal map 
	void DrawDiffNM(D3DXMATRIX* pMatrix, ID3DXEffect* pFX, D3DXHANDLE phNormalMap, D3DXHANDLE phMtrl, D3DXHANDLE phTex);

	//Draw mesh with environment map blending (diffuse + environment map)
	void DrawEnvBlnd(D3DXMATRIX* pMatrix, ID3DXEffect* pFX, D3DXHANDLE phMtrl, D3DXHANDLE phTex, D3DXHANDLE phTex1);

	//Add mesh animation function (arbitary rotations around the positive x-axis)
	void AnimRotX(float fTime);

private:
	//private mesh class data members
	ID3DXMesh* mMesh;
	ID3DXMesh* mLevel;
	std::vector<Mtrl> mMtrl;
	std::vector<IDirect3DTexture9*> mTex;
	IDirect3DTexture9* mDefTex;
	IDirect3DTexture9* mLevelTex;
	IDirect3DTexture9* mNMTex;
	IDirect3DCubeTexture9* mEnvMap;
};

extern XMesh* gMesh;
extern XMesh* gLevel;

#endif