//Scene interface
//All the rendering and shader stuff gets handled here

#ifndef _SCENE_H_
#define _SCENE_H_

#include "input.h"
#include "game.h"
#include "d3dwindow.h"

//Directional light struct
struct DirLight
{
	D3DXCOLOR ambient;
	D3DXCOLOR diffuse;
	D3DXCOLOR spec;
	D3DXVECTOR3 dirW;
};

//Genereic material struct
struct Mtrl
{
	Mtrl()
		:ambient(255), diffuse(255), spec(255), specPower(8.0f){}
	Mtrl(const D3DXCOLOR& a, const D3DXCOLOR& d, 
		 const D3DXCOLOR& s, float power)
		:ambient(a), diffuse(d), spec(s), specPower(power){}

	D3DXCOLOR ambient;
	D3DXCOLOR diffuse;
	D3DXCOLOR spec;
	float specPower;
};

//Water plane init info struct
struct WaterInitInfo
{
	DirLight dirLight;
	Mtrl     mtrl;
	int      vertRows;
	int      vertCols;
	float    dx;
	float    dz;
	char*	 waveMapFilename0;
	char*	 waveMapFilename1;
	D3DXVECTOR2 waveMapVelocity0;
	D3DXVECTOR2 waveMapVelocity1;
	float texScale;
	D3DXMATRIX toWorld;
};


class Scene 
{
public:

	//Class constructor
	Scene();

	//Class destructor
	~Scene();

	void SceneOnLostDevice();

	void SceneOnResetDevice();

	void UpdateScene(float dTime);

	//Main scene drawing function
	void Draw(D3DXMATRIX pMatrix, D3DXVECTOR3 CamPos);

	//additional functions for vertex construction
	void BuildFX();

private:

	//WVP matrices
	D3DXMATRIX mWorld;
	D3DXMATRIX mWorldInv;
	D3DXMATRIX mView;
	D3DXMATRIX mProj;

	//Effect handles
	ID3DXEffect*	mFX0; //effect handle for Diffuse + Ambient + Specular + Texture Phong (per-pixel) shader
	ID3DXEffect*	mFX1; //effect handle for normal map shader

	//effect technique handle (add more as required)
	D3DXHANDLE	 mhFXTech0; //Evironment mapping
	D3DXHANDLE	 mhFXTech1; //NM + diffuse

	D3DXHANDLE   mhWVP;
	D3DXHANDLE   mhWorldInvTrans;
	D3DXHANDLE   mhEyePos;
	D3DXHANDLE	 mhEyePosW;
	D3DXHANDLE   mhWorld;

	//mFX1 specific
	D3DXHANDLE   mhWVPfx1;
	D3DXHANDLE	 mhWorldInv;
	D3DXHANDLE   mhMtrlfx1;
	D3DXHANDLE   mhLightfx1;
	D3DXHANDLE	 mhEyePosWfx1;
	D3DXHANDLE   mhTexfx1;



	D3DXHANDLE   mhMtrl;

	D3DXHANDLE	 mhEnvMap; //env map
	D3DXHANDLE	 mhReflectivity; //env map

	D3DXHANDLE   mhNormalMap; //normal map

	D3DXHANDLE   mhTex; //dif tex

	D3DXHANDLE   mhLight;

	DirLight mLight;
};

extern Scene* gScene;

#endif