//Skysphere interface implementation

#include "sky.h"

//Sky class constructor implementation
Sky::Sky(const char * envmapFilename, float skyRadius)
{
	HR(D3DXCreateSphere(gd3dDevice, skyRadius, 30, 30, &mSphere, 0));
	HR(D3DXCreateCubeTextureFromFile(gd3dDevice, envmapFilename, &mSkyMap));

	ID3DXBuffer* errors = 0;
	HR(D3DXCreateEffectFromFile(gd3dDevice, "shaders/sky.c", 0, 0, 0, 
		0, &mSkyFX, &errors));

	if(errors)
	{
		MessageBox(0, (char*)errors->GetBufferPointer(), 0, 0);
	}

	mhTech   = mSkyFX->GetTechniqueByName("SkyTech");
	mhWVP    = mSkyFX->GetParameterByName(0, "gWVP");
	mhEnvMap = mSkyFX->GetParameterByName(0, "gEnvMap");

	// Set effect parameters that do not vary.
	HR(mSkyFX->SetTechnique(mhTech));
	HR(mSkyFX->SetTexture(mhEnvMap, mSkyMap));
}

//Sky class destructor implementation
Sky::~Sky()
{
	Release(mSphere);
	Release(mSkyMap);
	Release(mSkyFX);
}

void Sky::OnLostDevice()
{
	HR(mSkyFX->OnLostDevice());
}

void Sky::OnResetDevice()
{
	HR(mSkyFX->OnResetDevice());
}

IDirect3DCubeTexture9* Sky::getEnvMap()
{
	return mSkyMap;
}

float Sky::getRadius()
{
	return mRadius;
}

void Sky::draw(D3DXMATRIX pMatrix, D3DXVECTOR3 CamPos)
{
	// Sky always centered about camera's position.
	D3DXMATRIX W;
	D3DXMatrixTranslation(&W, CamPos.x, CamPos.y, CamPos.z);
	HR(mSkyFX->SetMatrix(mhWVP, &(W * pMatrix)));
	
	UINT numPasses = 0;
	HR(mSkyFX->Begin(&numPasses, 0));
	HR(mSkyFX->BeginPass(0));
		HR(mSphere->DrawSubset(0));
	HR(mSkyFX->EndPass());
	HR(mSkyFX->End());
}

