//Timer interface implementation

#include "timer.h"


//Timer constructor implementation
Timer::Timer()
: mTimerFont(NULL), mFPS(0.0f), mMSPerFrame(0.0f)
{
	//Setup the font for the timer
	D3DXFONT_DESC fontDesc;
	fontDesc.Height          = 18;
    fontDesc.Width           = 0;
    fontDesc.Weight          = 0;
    fontDesc.MipLevels       = 1;
    fontDesc.Italic          = false;
    fontDesc.CharSet         = DEFAULT_CHARSET;
    fontDesc.OutputPrecision = OUT_DEFAULT_PRECIS;
    fontDesc.Quality         = DEFAULT_QUALITY;
    fontDesc.PitchAndFamily  = DEFAULT_PITCH | FF_DONTCARE;
    strcpy_s(fontDesc.FaceName, _T("OCR-B 10 BT"));

	HR(D3DXCreateFontIndirect(gd3dDevice, &fontDesc, &mTimerFont));
}

//Timer destructor implementation
Timer::~Timer()
{
	//release font object
	Release(mTimerFont);
}

//ID3DXFont specific on lost device function implementation
void Timer::TimerOnLostDevice()
{
	mTimerFont->OnLostDevice(); //debug me
}

//ID3DXFont specific on reset device function implementation
void Timer::TimerOnResetDevice()
{
	mTimerFont->OnResetDevice(); //debug me
}

//Update timer function implementation
void Timer::TimerUpdate(float dt)
{
	static float numFrames   = 0.0f;
	static float timeElapsed = 0.0f;

	numFrames += 1.0f;
	timeElapsed += dt;

	if(timeElapsed >= 1.0f)
	{

		mFPS = numFrames;
		mMSPerFrame = 1000.0f / mFPS;

		timeElapsed = 0.0f;
		numFrames   = 0.0f;
	}
}

//Display realtime stats function implementation
void Timer::TimerDisplay()
{
	// Make static so memory is not allocated every frame.
	static char buffer[256];

	sprintf_s(buffer, "Frames Per Second = %.2f\n"
					"Milliseconds Per Frame = %.4f\n"
					, mFPS, mMSPerFrame);

	RECT R = {5, 5, 0, 0};
	HR(mTimerFont->DrawText(0, buffer, -1, &R, DT_NOCLIP, D3DCOLOR_XRGB(255,0,255)));
}